"use strict";

var _react = _interopRequireDefault(require("react"));

var _reactDom = _interopRequireDefault(require("react-dom"));

var _testUtils = require("react-dom/test-utils");

var _useCssHook = require("./use-css.hook.js");

var _styleElementUtils = require("./style-element-utils.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var numRenders = 0;
describe('useCss()', function () {
  var container;
  beforeEach(function () {
    (0, _styleElementUtils.resetState)();
    container = document.createElement('div');
    numRenders = 0;
    var styleElements = document.querySelectorAll('style');
    Array.prototype.slice.call(document.querySelectorAll('style')).forEach(function (styleElement) {
      styleElement.remove();
    });
  });
  it('returns the correct props for a dom react element', function () {
    var css = "& .foo {}";

    _reactDom.default.render(_react.default.createElement(ScopedDiv, {
      css: css
    }), container);

    expect(container.innerHTML).toEqual('<div data-kremling="0"></div>');

    _reactDom.default.unmountComponentAtNode(container);
  });
  it('creates a <style> element when mounting new css, and removes the <style> element when unmounting the component', function () {
    var css = "& .foo {}";
    (0, _testUtils.act)(function () {
      _reactDom.default.render(_react.default.createElement(ScopedDiv, {
        css: css
      }), container);
    });
    var styleElement = document.querySelector('style');
    expect(styleElement).not.toBeUndefined();
    expect(styleElement.textContent).toEqual("[data-kremling='0'] .foo, [data-kremling='0'].foo {}");
    expect(styleElement.isConnected).toBe(true);
    (0, _testUtils.act)(function () {
      _reactDom.default.unmountComponentAtNode(container);
    });
    expect(styleElement.isConnected).toBe(false);
  });
  it('reuses an existing style element from a different instance or component', function () {
    var css = "& .foo {}";
    var existingStyleElement = document.createElement('style');
    existingStyleElement.type = 'text/css';
    existingStyleElement.textContent = "[data-kremling='4'] .foo, [data-kremling='4'].foo {}";
    existingStyleElement.kremlings = 1;
    existingStyleElement.kremlingAttr = 'data-kremling';
    existingStyleElement.kremlingValue = 4;
    document.head.appendChild(existingStyleElement);
    _styleElementUtils.styleTags[css] = existingStyleElement;
    (0, _testUtils.act)(function () {
      _reactDom.default.render(_react.default.createElement(ScopedDiv, {
        css: css
      }), container);
    });
    expect(document.querySelectorAll('style').length).toBe(1);
    expect(document.querySelector('style')).toBe(existingStyleElement);
    expect(container.innerHTML).toEqual('<div data-kremling="4"></div>'); // The style element is in the dom

    expect(existingStyleElement.isConnected).toBe(true);
    expect(existingStyleElement.kremlings).toBe(2);
    (0, _testUtils.act)(function () {
      _reactDom.default.unmountComponentAtNode(container);
    }); // The style element is still in the dom because there was a previous thing using it

    expect(existingStyleElement.isConnected).toBe(true);
    expect(existingStyleElement.kremlings).toBe(1);
  });
  it("doesn't cause the component to render more than once", function () {
    var css = "& .foo {}";
    (0, _testUtils.act)(function () {
      _reactDom.default.render(_react.default.createElement(ScopedDiv, {
        css: css
      }), container);
    });
    expect(numRenders).toBe(1);
    (0, _testUtils.act)(function () {
      _reactDom.default.unmountComponentAtNode(container);
    });
  });
  it("doesn't do anything weird on subsequent renders", function () {
    var css = "& .foo {}";
    (0, _testUtils.act)(function () {
      _reactDom.default.render(_react.default.createElement(ScopedDiv, {
        css: css
      }), container);
    });
    expect(document.querySelectorAll('style').length).toBe(1);
    var styleElement = document.querySelector('style');
    expect(styleElement.kremlings).toBe(1);
    expect(styleElement.kremlingAttr).toBe('data-kremling');
    expect(styleElement.kremlingValue).toBe(0); // Subsequent render

    (0, _testUtils.act)(function () {
      _reactDom.default.render(_react.default.createElement(ScopedDiv, {
        css: css
      }), container);
    });
    expect(document.querySelectorAll('style').length).toBe(1);
    styleElement = document.querySelector('style');
    expect(styleElement.kremlings).toBe(1);
    expect(styleElement.kremlingAttr).toBe('data-kremling');
    expect(styleElement.kremlingValue).toBe(0);
    (0, _testUtils.act)(function () {
      _reactDom.default.unmountComponentAtNode(container);
    });
  });
  it("allows you to change the dom attribute with a namespace argument", function () {
    var css = "& .foo {}";
    (0, _testUtils.act)(function () {
      _reactDom.default.render(_react.default.createElement(ScopedDiv, {
        css: css,
        namespace: "yoshi"
      }), container);
    });
    expect(container.innerHTML).toEqual('<div data-yoshi="0"></div>');
    (0, _testUtils.act)(function () {
      _reactDom.default.unmountComponentAtNode(container);
    });
  });
  it("works with postcss", function () {
    var postcss = {
      id: '15',
      styles: "[donkey-kong='15'] .foo, [donkey-kong='15'].foo {}",
      namespace: 'donkey-kong'
    };
    (0, _testUtils.act)(function () {
      _reactDom.default.render(_react.default.createElement(ScopedDiv, {
        css: postcss
      }), container);
    });
    expect(container.innerHTML).toEqual('<div donkey-kong="15"></div>');
    (0, _testUtils.act)(function () {
      _reactDom.default.unmountComponentAtNode(container);
    });
  });
});

function ScopedDiv(props) {
  var scope = (0, _useCssHook.useCss)(props.css, props.namespace);
  numRenders++;
  return _react.default.createElement("div", scope);
}
//# sourceMappingURL=use-css.test.js.map